#
# MicroTESK for PowerPC
#
# Copyright (c) 2016-2018 Institute for System Programming of the Russian Academy of Sciences
# All Rights Reserved
#
# Institute for System Programming of the Russian Academy of Sciences (ISP RAS)
# 25 Alexander Solzhenitsyn st., Moscow, 109004, Russia
# http://www.ispras.ru
#

require_relative 'powerpc_base'

#
# Description:
#
# This test template demonstrates how MicroTESK can simulate the execution
# of a test program to predict the resulting state of a microprocessor
# design under test. The described test program is a simple implemention of 
# the bubble sort algorithm. The algorithm in pseudocode (from Wikipedia):
#
# procedure bubbleSort( A : list of sortable items )
#   n = length(A)
#   repeat 
#     swapped = false
#     for i = 1 to n-1 inclusive do
#       /* if this pair is out of order */
#       if A[i-1] > A[i] then
#         /* swap them and remember something changed */
#         swap( A[i-1], A[i] )
#         swapped = true
#       end if
#     end for
#   until not swapped
# end procedure
#
class BubbleSortWordTemplate < PowerPCBaseTemplate
  def pre
    super

    data {
      label :data
      word rand(1, 9), rand(1, 9), rand(1, 9), rand(1, 9)
      word rand(1, 9), rand(1, 9), rand(1, 9), rand(1, 9)
      word rand(1, 9), rand(1, 9), rand(1, 9), rand(1, 9)
      label :end
      space 1
    }
  end

  def run
    #trace_data :data, :end
    addi r1, r0, 0x0fc0
    trace "Start address = 0x%x", gpr_observer(1)
    addi r2, r0, rand(1, 15)
    stw r2, 0x0, r1
    trace "Store value: 0x%x address:[0x%x]", gpr_observer(2), gpr_observer(1)
    addi r3, r1, 0x4
    addi r2, r0, rand(1, 15)
    stw r2, 0x0, r3
    trace "Store value: 0x%x address:[0x%x]", gpr_observer(2), gpr_observer(3)
    6.times {
      addi r3, r3, 0x4
      addi r2, r0, rand(1, 15)
      stw r2, 0x0, r3
      trace "Store value: 0x%x address:[0x%x]", gpr_observer(2), gpr_observer(3)
    }

    #la s0, :data
    #la s1, :end
    addi r2, r1, 0x0 #start
    addi r3, r2, 0x20 # end

    Or r5, r0, r0
    ########################### Outer loop starts ##############################
    label :repeat
    #b :for

    addi r4, r2, 4
    ########################### Inner loop starts ##############################
    label :for
    # TODO: branch instruction

    #beq r4, r3, :exit_for
    #daddi t2, r4, -4

    #lw t4, 0, r4
    #lw t5, 0, t2

    #slt t6, t4, t5
    #beq t6, r0, :next
    #nop

    #nor r5, r0, r0

    #sw t4, 0, t2
    #sw t5, 0, r4

    #label :next
    #j :for
    #daddi r4, r4, 4
    ############################ Inner loop ends ###############################
    label :exit_for

    #bne r5, r0, :repeat
    #Or r5, r0, r0
    ############################ Outer loop ends ###############################

    #trace_data :data, :end
  end

end
