#
# MicroTESK PowerPC Edition
#
# Copyright (c) 2016 Institute for System Programming of the Russian Academy of Sciences
# All Rights Reserved
#
# Institute for System Programming of the Russian Academy of Sciences (ISP RAS)
# 25 Alexander Solzhenitsyn st., Moscow, 109004, Russia
# http://www.ispras.ru
#

require_relative 'powerpc_base'

#
# Description:
#
# Test of disassembled abs functions.
# 
# int abs1(int n)
# {
#    int t = (unsigned int)n >> (sizeof(int)* 8 -1);
#    return ( -t) ^ (n-t);
# }
#
# int abs2(int j)
# {
#	return (j >= 0) ? j : -j;
# } 

class AbsTest < PowerPCBaseTemplate

  def run
    trace "Run abs test:"
    trace "\nPowerPC abs test:\n"
    
    # abs1 test 1: x = -0x10
    trace "abs1 test 1: x = -0x10:\n"
    
    addi    r1, r0, 0x0fff  # set stack
    addi    r3, r0, -0x10     # x = r3 = -0x10     
    
    trace "r1 = %x", gpr_observer(1)
    trace "r3 = %x", gpr_observer(3)
    
    stwu    r1, -0x30, r1 # Alternative name is '__partition_begin'
    stw     r31, 0x2C, r1
    mr      r31, r1
    stw     r3, 0x18, r31
    lwz     r9, 0x18, r31
    trace "r9 = %x", gpr_observer(9)
    trace "r31 = %x", gpr_observer(31)
    srwi    r9, r9, 31
    stw     r9, 0x8, r31
    lwz     r9, 0x8, r31
    neg     r10, r9
    lwz     r8, 0x18, r31
    lwz     r9, 0x8, r31
    subf    r9, r9, r8
    xor     r9, r10, r9
    mr      r3, r9
    addi    r11, r31, 0x30
    lwz     r31, -0x4, r11
    mr      r1, r11
    
    trace "r1 = %x", gpr_observer(1)
    trace "r3 = %x", gpr_observer(3)
    
    # abs1 test 2: x = 0x10
    trace "\nabs1 test 2: x = 0x10:\n"
    
    addi    r1, r0, 0x0fff  # set stack
    addi    r3, r0, 0x10     # x = r3 = 0x10     
    
    trace "r1 = %x", gpr_observer(1)
    trace "r3 = %x", gpr_observer(3)
    
    stwu    r1, -0x30, r1 # Alternative name is '__partition_begin'
    stw     r31, 0x2C, r1
    mr      r31, r1
    stw     r3, 0x18, r31
    lwz     r9, 0x18, r31
    trace "r9 = %x", gpr_observer(9)
    trace "r31 = %x", gpr_observer(31)
    srwi    r9, r9, 31
    stw     r9, 0x8, r31
    lwz     r9, 0x8, r31
    neg     r10, r9
    lwz     r8, 0x18, r31
    lwz     r9, 0x8, r31
    subf    r9, r9, r8
    xor     r9, r10, r9
    mr      r3, r9
    addi    r11, r31, 0x30
    lwz     r31, -0x4, r11
    mr      r1, r11
    
    trace "r1 = %x", gpr_observer(1)
    trace "r3 = %x", gpr_observer(3)
    
    # abs2 test 1: x = -0x10
    trace "\nabs2 test 1: x = -0x10:\n"
    
    addi    r1, r0, 0x0fff  # set stack
    addi    r3, r0, -0x10     # x = r3 = -0x10     
    
    trace "r1 = %x", gpr_observer(1)
    trace "r3 = %x", gpr_observer(3)
    
    stwu    r1, -0x20, r1
    stw     r31, 0x1C, r1
    mr      r31, r1
    stw     r3, 8, r31
    lwz     r9, 8, r31
    trace "r9 = %x", gpr_observer(9)
    srawi   r10, r9, 0x1F
    trace "r10 = %x", gpr_observer(10)
    lwz     r9, 8, r31
    trace "r9 = %x", gpr_observer(9)
    xor     r9, r10, r9
    trace "r9 = %x", gpr_observer(9)
    subf    r9, r10, r9
    trace "r9 = %x", gpr_observer(9)
    mr      r3, r9
    trace "r3 = %x", gpr_observer(3)
    addi    r11, r31, 0x20
    lwz     r31, -4, r11
    mr      r1, r11
    
    trace "r1 = %x", gpr_observer(1)
    trace "r3 = %x", gpr_observer(3)
    
    # abs2 test 2: x = 0x10
    trace "\nabs2 test 2: x = 0x10:\n"
    
    addi    r1, r0, 0x0fff  # set stack
    addi    r3, r0, 0x10     # x = r3 = 0x10     
    
    trace "r1 = %x", gpr_observer(1)
    trace "r3 = %x", gpr_observer(3)
    
    stwu    r1, -0x20, r1
    stw     r31, 0x1C, r1
    mr      r31, r1
    stw     r3, 8, r31
    lwz     r9, 8, r31
    srawi   r10, r9, 0x1F
    lwz     r9, 8, r31
    xor     r9, r10, r9
    subf    r9, r10, r9
    mr      r3, r9
    addi    r11, r31, 0x20
    lwz     r31, -4, r11
    mr      r1, r11
    
    trace "r1 = %x", gpr_observer(1)
    trace "r3 = %x", gpr_observer(3)
  end

end