#
# MicroTESK for PowerPC
#
# Copyright (c) 2016-2018 Institute for System Programming of the Russian Academy of Sciences
# All Rights Reserved
#
# Institute for System Programming of the Russian Academy of Sciences (ISP RAS)
# 25 Alexander Solzhenitsyn st., Moscow, 109004, Russia
# http://www.ispras.ru
#

require_relative 'powerpc_base'

#
# Description:
#
# This small tests for MMU instructions.
#

class InstructionMMU < PowerPCBaseTemplate

  def run
    trace "Run MMU instruction:"
    trace "\nPowerPC Load/Store:\n"

    sequence {
      trace "Little-Endian tests:"
      addi r1, r0, 0x0fff
      trace "r1 = 0x%x", gpr_observer(1)
      addi r2, r0, 0xdead
      trace "r2 = 0x%x", gpr_observer(2)
      slwi r2, r2, 16
      trace "r2 = 0x%x", gpr_observer(2)
      ori r2, r2, 0xbeaf
      trace "r2 = 0x%x", gpr_observer(2)
      stw r2, 0x0, r1
      trace "r1 = 0x%x", gpr_observer(1)
      trace "r2 = 0x%x", gpr_observer(2)
      lwz r3, 0x0, r1
      trace "r3 = 0x%x", gpr_observer(3)
      lbz r3, 0x0, r1
      trace "r3 = 0x%x", gpr_observer(3)
      trace "r1 = 0x%x", gpr_observer(1)
      lbz r3, 0x1, r1
      trace "r3 = 0x%x", gpr_observer(3)
      trace "r1 = 0x%x", gpr_observer(1)
      lbz r3, 0x2, r1
      trace "r3 = 0x%x", gpr_observer(3)
      lbz r3, 0x3, r1
      trace "r3 = 0x%x", gpr_observer(3)
      stb r2, 0x0, r1
      stb r2, 0x1, r1
      stb r2, 0x2, r1
      stb r2, 0x3, r1
      lwz r3, 0x0, r1
      trace "r3 = 0x%x", gpr_observer(3)
    }.run

    addi r1, r0, 0x0fff
    lwz r2, -0x30,r1
    stwu r1, -0x30, r1
    trace "r1 = %x", gpr_observer(1)
    addi r2, r0, 0x123
    addi r5, r0, 0x0f00

    stw r2, 0x0, r1
    trace "r2 = %x", gpr_observer(2)
    stw r5, 0x4, r1
    trace "r5 = %x", gpr_observer(5)

    lwz r3, 0x0, r1
    trace "r3 = %x", gpr_observer(3)
    lwzu r4, 0x4, r1
    trace "r4 = %x", gpr_observer(4)
    trace "r1 = %x", gpr_observer(1)
    
    li r2,0x8
    stwux r5,r2,r5
    trace "r2 = %x", gpr_observer(2)
    stwx r2,r2,r5
    li r2,0x8
    lwzux r3, r2, r5
    trace "r3 = %x", gpr_observer(3)
    lwzux r3, r2, r5
    trace "r3 = %x", gpr_observer(3)
    addis r3,r3,0xABCD
    stw r3,0,r5
    lbzu r3,1,r5
    trace "r3 = %x", gpr_observer(3)
    lbz r3, -1, r5
    trace "r3 = %x", gpr_observer(3)
    li r2,1
    lbzux r3,r5,r2
    trace "r3 = %x", gpr_observer(3)
    lbzx r3,r5,r2
    trace "r3 = %x", gpr_observer(3)
    stbu r3,1,r5
    stb r3,1,r5
    stbux r3,r5,r2
    stbx r3,r5,r2
    lwz r3,-4,r5
    trace "r3 = %x", gpr_observer(3)
    lhzu r3,-3,r5
    trace "r3 = %x", gpr_observer(3)
    lhz r3,1,r5
    trace "r3 = %x", gpr_observer(3)
    lhzux r3,r5,r2
    trace "r3 = %x", gpr_observer(3)
    lhzx r3,r5,r2
    trace "r3 = %x", gpr_observer(3)
    sthu r3,1,r5
    sth r3,1,r5
    sthux r3,r5,r2
    sthx r3,r5,r2
    lwz r3,-2,r5
    trace "r3 = %x", gpr_observer(3)
    lhau r3,-5,r5
    trace "r3 = %x", gpr_observer(3)
    lha r3,1,r5
    trace "r3 = %x", gpr_observer(3)
    lhaux r3,r5,r2
    trace "r3 = %x", gpr_observer(3)
    lhax r3,r5,r2
    trace "r3 = %x", gpr_observer(3)
  end

end